"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ayax_common_cache_1 = require("ayax-common-cache");
class AuthService {
    constructor(options) {
        this._tokenStorageItem = "token";
        this._accessRules = "accessRules";
        this._modulesStorageItem = "modules";
        this._currentUserStorageItem = "currentUser";
        this._identityOperation = options.identityOperation;
        this._readerOperation = options.readerOperation;
        this._cacheExpiresAfter = options.cacheExpiresAfter ? options.cacheExpiresAfter : 15;
        this._authenticateUrl = options.authenticateUrl ? options.authenticateUrl : "/authentication/Login";
        this._token = options.token;
        this._cookieDomain = location.href.startsWith("http://localhost")
            ? "localhost"
            : options.cookieDomain
                ? options.cookieDomain
                : window.location.hostname.split(".").slice(-2).join(".");
        this._tokenExpiresInHours = options.tokenExpiresInHours ? options.tokenExpiresInHours : 72;
        this._cookieTokenName = options.cookieTokenName ? options.cookieTokenName : "token";
        if (options.modules) {
            this.modules = options.modules;
        }
    }
    get accessRules() {
        const fromStorage = localStorage.getItem(this._accessRules);
        if (fromStorage) {
            return JSON.parse(fromStorage);
        }
        else {
            return null;
        }
    }
    set accessRules(value) {
        localStorage.setItem(this._accessRules, JSON.stringify(value));
    }
    get modules() {
        const fromStorage = localStorage.getItem(this._modulesStorageItem);
        if (fromStorage) {
            return JSON.parse(fromStorage);
        }
        else {
            return null;
        }
    }
    set modules(value) {
        localStorage.setItem(this._modulesStorageItem, JSON.stringify(value));
    }
    get currentUser() {
        if (!this._currentUser) {
            const currentUserFromStorage = localStorage.getItem(this._currentUserStorageItem);
            this._currentUser = JSON.parse(currentUserFromStorage);
        }
        return this._currentUser;
    }
    set currentUser(value) {
        const currentUserFromStorage = localStorage.getItem(this._currentUserStorageItem);
        if (!currentUserFromStorage) {
            localStorage.setItem(this._currentUserStorageItem, JSON.stringify(value));
        }
        this._currentUser = value;
    }
    GetCurrentUser() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.currentUser) {
                return this.currentUser;
            }
            try {
                const currentUserFromLocalStorage = localStorage.getItem(this._currentUserStorageItem);
                if (currentUserFromLocalStorage) {
                    const value = JSON.parse(currentUserFromLocalStorage);
                    this.currentUser = value;
                    return value;
                }
                const currentUser = yield this.GetAuthenticatedUser();
                this.currentUser = currentUser;
                return currentUser;
            }
            catch (e) {
                localStorage.clear();
                throw new Error(e);
            }
        });
    }
    Login(login, password, modules) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!login || !password) {
                console.error("Неверные параметры для авторизации");
                return false;
            }
            if (modules) {
                this.modules = modules;
            }
            const request = {
                login,
                password,
            };
            if (modules) {
                request["modules"] = modules;
            }
            try {
                const operation = yield this._identityOperation
                    .post(this._authenticateUrl, request)
                    .then(x => x.ensureSuccess());
                const result = operation;
                this._token = result.token;
                localStorage.setItem(this._tokenStorageItem, result.token);
                localStorage.setItem(this._accessRules, JSON.stringify(result.accessRules));
                localStorage.setItem(this._modulesStorageItem, JSON.stringify(modules));
                this.SetTokenCookie(result.token);
                yield this.GetCurrentUser();
                return true;
            }
            catch (e) {
                localStorage.clear();
                this.DeleteTokenCookie();
                console.error(`Ошибка авторизации ${JSON.stringify(e)}`);
                return false;
            }
        });
    }
    CheckAuthentication(login) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = { token: this._token, modules: this.modules };
            try {
                this.DeleteTokenCookie();
                this.currentUser = yield this._identityOperation
                    .post(`/authentication/GetAuthenticatedUser`, request)
                    .then(x => x.ensureSuccess());
                if (login && this.currentUser.login.toLocaleLowerCase() !== login.toLocaleLowerCase()) {
                    console.error("Токен не соотвествует пользователю");
                    this.DeleteTokenCookie();
                    return false;
                }
                this.SetTokenCookie(this._token);
            }
            catch (e) {
                console.error(e);
                this.DeleteTokenCookie();
                return false;
            }
            return true;
        });
    }
    GetAuthenticatedUser(modules) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._token || this._token === "") {
                console.error(`Неверный токен token=${this._token}`);
                throw Error(`Ошибка авторизации`);
            }
            const request = { token: this._token };
            if (modules && modules.length > 0) {
                request["modules"] = modules;
                this.modules = modules;
            }
            else if (this.modules) {
                request["modules"] = this.modules;
            }
            try {
                const user = yield this._identityOperation
                    .post(`/authentication/GetAuthenticatedUser`, request)
                    .then(x => x.ensureSuccess());
                this.currentUser = user;
                if (!this.accessRules && user.accessRulesNames.length > 0) {
                    this.accessRules = user.accessRulesNames;
                    location.reload();
                }
                return user;
            }
            catch (e) {
                console.error(e);
                throw Error(e);
            }
        });
    }
    GetUserById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!id) {
                return null;
            }
            const operation = yield this._readerOperation.get(`/user/getuserbyid/${id}`);
            if (operation.status === 0) {
                const user = operation.result;
                return yield this.GetUserByUid(`${user.uid}`);
            }
            else {
                console.error(operation.message);
                throw Error(`Ошибка загрузки ${operation.message}`);
            }
        });
    }
    GetUserByUid(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            const operation = yield this._readerOperation.get(`/user/getuserbyuid/${uid}`);
            if (operation.status === 0) {
                const user = operation.result;
                return user;
            }
            else {
                console.error(operation.message);
                throw Error(`Ошибка загрузки ${operation.message}`);
            }
        });
    }
    GetUsers(subdivisionId) {
        return __awaiter(this, void 0, void 0, function* () {
            return subdivisionId ? this.GetUsersForSubdivisionList([subdivisionId]) : this.GetUsersForSubdivisionList();
        });
    }
    GetUsersForSubdivisionList(subdivisionsIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = subdivisionsIds
                ? { page: 1, perPage: 10000, subdivisionsIds, showChildren: true }
                : { page: 1, perpage: 10000 };
            const fetchResponse = () => this._readerOperation
                .search("/user/search", request)
                .then(x => x.ensureSuccess())
                .then(x => x.data);
            return ayax_common_cache_1.CacheHelper.TryFromCache(fetchResponse, this._cacheExpiresAfter, "post", "/user/search", request);
        });
    }
    GetSubdivisions(isMain) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = {};
            if (isMain) {
                request["isMain"] = isMain;
            }
            else {
                request["isMain"] = true;
            }
            const fetchResponse = () => this._readerOperation
                .search("/subdivision/search", request)
                .then(x => x.ensureSuccess())
                .then(x => x.data);
            return ayax_common_cache_1.CacheHelper.TryFromCache(fetchResponse, this._cacheExpiresAfter, "post", "/subdivision/search", request);
        });
    }
    GetUsersForMyDivision() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.GetUsersForSubdivisionList(this.currentUser.subdivisionIds);
        });
    }
    SetTokenCookie(token) {
        let expires = "";
        const date = new Date();
        date.setTime(date.getTime() + this._tokenExpiresInHours * 60 * 1000);
        expires = "; expires=" + date.toUTCString();
        document.cookie = `${this._cookieTokenName}=${token}${expires};path=/;domain=${this._cookieDomain};`;
    }
    DeleteTokenCookie() {
        document.cookie = `${this._cookieTokenName}=; Max-Age=-99999999;path=/;domain=${this._cookieDomain};`;
    }
    SearchUsers(request) {
        const fetchResponse = () => this._readerOperation
            .search("/user/search", request)
            .then(x => x.ensureSuccess())
            .then(x => x.data);
        return ayax_common_cache_1.CacheHelper.TryFromCache(fetchResponse, this._cacheExpiresAfter, "post", "/user/search", request);
    }
    GetGroup(guid) {
        return this._readerOperation
            .get(`/group/getgroupbyuid/${guid}`)
            .then(x => x.ensureSuccess());
    }
    GetGroupUsers() {
        return this._readerOperation
            .get(`/group/getgroupusers`)
            .then(x => x.ensureSuccess());
    }
    SearchGroups(request) {
        const fetchResponse = () => this._readerOperation
            .search("/group/search", request)
            .then(x => x.ensureSuccess())
            .then(x => x.data);
        return ayax_common_cache_1.CacheHelper.TryFromCache(fetchResponse, this._cacheExpiresAfter, "post", "/group/search", request);
    }
}
exports.AuthService = AuthService;
